{% spaceless %}<div{% include "django/forms/widgets/attrs.html" %}>{% for widget in widget.subwidgets %}{% include widget.template_name %}{% if not forloop.last %}&nbsp;~&nbsp;{% endif %}{% endfor %}</div>{% endspaceless %}
