<h3>{{ title }}: </h3>
<ul>
    <li>
    <select style="width: 100%;"
            onchange='django.jQuery("#changelist-filter input, #changelist-filter select").attr("disabled", "disabled");window.location = window.location.pathname + this.options[this.selectedIndex].value'>
    {% for choice in choices %}
        <option{% if choice.selected %} selected="selected"{% endif %}
             value="{{ choice.query_string|iriencode }}">{{ choice.display }}</option>
    {% endfor %}
    </select>
    </li>
</ul>
